/*      */ package com.medicwave.cardgame.poker;
/*      */ 
/*      */ import ca.ualberta.cs.poker.Card;
/*      */ import ca.ualberta.cs.poker.Deck;
/*      */ import ca.ualberta.cs.poker.Hand;
/*      */ import com.medicwave.socketlibrary.SocketServer;
/*      */ import java.io.IOException;
/*      */ import java.util.Enumeration;
/*      */ import java.util.Vector;
/*      */ import se.lypson.common.IComparator;
/*      */ import se.lypson.common.Utilities;
/*      */ import se.lypson.common.collections.IForwardIterator;
/*      */ import se.lypson.common.collections.LinkedList;
/*      */ import se.lypson.common.collections.OrderedCountingSet;
/*      */ import se.lypson.common.thread.Thread;
/*      */ 
/*      */ public class PokerServer extends Thread
/*      */ {
/*      */   private static final String SPACE_STRING = " ";
/*      */   public static final int MAXIMUM_NUMBER_OF_PLAYERS = 5;
/*      */   private static final int CARDS_IN_FULL_HAND = 5;
/*      */   private static final int CLIENT_ID_NO_CLIENT = -1;
/*      */   private final int numberOfPlayers;
/*      */   private final int startChips;
/*      */   private final int raiseAnteAfterRoundsCount;
/*      */   private final boolean raisingAnte;
/*      */   private final int maximumResponseTime;
/*      */   private SocketServer socketServer;
/*      */   private int startPlayerIndex;
/*      */   private Vector connectedPlayers;
/*      */   private Vector activePlayers;
/*      */   
/*      */   private class PotData
/*      */   {
/*      */     int playerInput;
/*      */     Vector players;
/*      */     
/*      */     public PotData(int playerInput)
/*      */     {
/*   40 */       this.players = new Vector(5);
/*   41 */       this.playerInput = playerInput;
/*      */     }
/*      */     
/*      */     public int getPotSize() {
/*   45 */       return this.playerInput * this.players.size();
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */     public void orderPlayersByHands()
/*      */     {
/*   52 */       Utilities.sort(this.players, PokerServer.this.getPlayerOrderByHandComparator());
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public int getNumberOfWinners()
/*      */     {
/*   61 */       if (this.players.size() <= 1)
/*   62 */         return this.players.size();
/*   63 */       int highestRank = ((ServerPlayerData)this.players.elementAt(0)).getRank();
/*   64 */       for (int i = 1; i < this.players.size(); i++)
/*      */       {
/*   66 */         if (((ServerPlayerData)this.players.elementAt(i)).getRank() != highestRank)
/*   67 */           return i;
/*      */       }
/*   69 */       return this.players.size();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*   91 */   private LinkedList gameOverPlayers = new LinkedList();
/*   92 */   private LinkedList disconnectedPlayers = new LinkedList();
/*   93 */   private Deck deckOfCards = new Deck();
/*      */   private int ante;
/*   95 */   private int maximumBet = 0;
/*   96 */   private int minimumRaise = 0;
/*      */   
/*   98 */   private long timeToSleep = 1000L;
/*      */   
/*  100 */   private volatile int expectingAnswerFromClientID = -1;
/*  101 */   private volatile String[] receivedAnswerFromClientID = null;
/*      */   
/*  103 */   private volatile boolean stop = false;
/*      */   
/*  105 */   private final Object lock = new Object();
/*      */   
/*  107 */   private Vector textReceivers = new Vector();
/*  108 */   private Vector gameObservers = new Vector();
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public PokerServer(int port, int numberOfClients, int playersStartChips, int ante, int raiseAnteAfterRoundsCount, int maximumResponseTime, int timeToSleep)
/*      */   {
/*  128 */     if (port < 0)
/*  129 */       throw new IllegalArgumentException("Port must be positive.");
/*  130 */     if ((numberOfClients < 2) || (numberOfClients > 5))
/*  131 */       throw new IllegalArgumentException("The number of players must be between 2 and 5.");
/*  132 */     if (playersStartChips <= 0)
/*  133 */       throw new IllegalArgumentException("The players must have a posititive amount of start chips.");
/*  134 */     if (ante < 0)
/*  135 */       throw new IllegalArgumentException("The ante cannot be negative.");
/*  136 */     if (maximumResponseTime <= 0)
/*  137 */       throw new IllegalArgumentException("The maximum response time must be strictly positive.");
/*  138 */     if (timeToSleep < 0) {
/*  139 */       throw new IllegalArgumentException("The time to sleep cannot be negative.");
/*      */     }
/*  141 */     this.numberOfPlayers = numberOfClients;
/*  142 */     this.startChips = playersStartChips;
/*  143 */     this.ante = ante;
/*  144 */     this.raiseAnteAfterRoundsCount = raiseAnteAfterRoundsCount;
/*  145 */     this.raisingAnte = (raiseAnteAfterRoundsCount > 0);
/*  146 */     this.maximumResponseTime = maximumResponseTime;
/*      */     
/*  148 */     this.connectedPlayers = new Vector(numberOfClients);
/*  149 */     this.activePlayers = new Vector(numberOfClients);
/*      */     
/*  151 */     this.socketServer = new SocketServer(port, numberOfClients);
/*  152 */     this.socketServer.addServerObserver(new com.medicwave.socketlibrary.ISocketServerObserver()
/*      */     {
/*      */       public void serverStartedEvent(SocketServer sender) {
/*      */         try {
/*  156 */           PokerServer.this.notifyTextReceivers("Poker server started.\nLocal address: " + sender.getLocalServerAddress() + "\n" + "Public address: " + sender.getPublicServerAddress() + "\n" + "Port: " + sender.getServerPort());
/*      */ 
/*      */         }
/*      */         catch (IOException ioe)
/*      */         {
/*  161 */           PokerServer.this.notifyTextReceivers("Server started, but there was an error retrieving the server address:\n" + ioe);
/*      */         }
/*      */       }
/*      */       
/*      */       public void clientConnectedEvent(SocketServer sender, int clientID) {
/*  166 */         PokerServer.this.notifyTextReceivers("Player #" + clientID + " connected. Requesting player name.");
/*      */         
/*  168 */         PokerServer.this.socketServer.sendMessage(clientID, "Name?");
/*      */       }
/*      */       
/*      */       public void clientDisconnectedEvent(SocketServer sender, int clientID) {
/*  172 */         PokerServer.this.notifyTextReceivers("Disconnected player #" + clientID + ".");
/*  173 */         PokerServer.this.getPlayerForClientID(clientID).setConnected(false);
/*      */       }
/*      */       
/*      */       public void messageSentEvent(SocketServer sender, String message) {
/*  177 */         PokerServer.this.notifyTextReceivers("Sent message '" + message + "' to all players.");
/*      */       }
/*      */       
/*      */       public void messageSentEvent(SocketServer sender, int clientID, String message) {
/*  181 */         PokerServer.this.notifyTextReceivers("Sent message '" + message + "' to player #" + clientID + ".");
/*      */       }
/*      */       
/*      */       public void messageReceivedEvent(SocketServer sender, int clientID, String message) {
/*  185 */         PokerServer.this.notifyTextReceivers("Received message '" + message + "' from player #" + clientID);
/*      */         try {
/*  187 */           String[] messageParts = Utilities.splitString(message, " ");
/*      */           
/*  189 */           if (clientID == PokerServer.this.expectingAnswerFromClientID)
/*      */           {
/*  191 */             synchronized (PokerServer.this.lock)
/*      */             {
/*  193 */               if (clientID == PokerServer.this.expectingAnswerFromClientID)
/*      */               {
/*  195 */                 PokerServer.this.notifyTextReceivers("Received expected message from client #" + clientID + ". Message is '" + message + "'.");
/*  196 */                 PokerServer.this.receivedAnswerFromClientID = messageParts;
/*  197 */                 PokerServer.this.lock.notify();
/*      */               }
/*      */             }
/*      */           }
/*  201 */           else if ((clientID != PokerServer.this.expectingAnswerFromClientID) && (messageParts[0].equals("Name")))
/*      */           {
/*  203 */             if (PokerServer.this.connectedPlayers.size() != PokerServer.this.numberOfPlayers)
/*      */             {
/*  205 */               PokerServer.this.addNewPlayer(clientID, messageParts[1]);
/*  206 */               PokerServer.this.notifyTextReceivers("Connected player '" + messageParts[1] + "' (client ID #" + clientID + ").");
/*      */               
/*  208 */               if (PokerServer.this.connectedPlayers.size() == PokerServer.this.numberOfPlayers) {
/*  209 */                 PokerServer.this.notifyServer();
/*      */               }
/*      */             } else {
/*  212 */               throw new IllegalStateException("Cannot receive new player connections after the maximum amount of players have connected.");
/*      */             }
/*      */           }
/*      */           else {
/*  216 */             PokerServer.this.notifyTextReceivers("Poker server: Unexpected message '" + message + "' from client #" + clientID + ".");
/*  217 */             throw new IllegalArgumentException("Unexpected message '" + message + "' from client #" + clientID + ".");
/*      */           }
/*      */         }
/*      */         catch (RuntimeException re) {
/*  221 */           PokerServer.this.notifyTextReceivers("Poker server: Error in messageReceivedEvent: " + re);
/*  222 */           throw re;
/*      */         }
/*      */       }
/*  225 */     });
/*  226 */     this.socketServer.addThreadObserver(new se.lypson.common.thread.IThreadObserver()
/*      */     {
/*      */       public void threadMessageEvent(Thread thread, Object message) {}
/*      */       
/*      */       public void threadFinishedEvent(Thread thread)
/*      */       {
/*  232 */         PokerServer.this.notifyTextReceivers("Poker Server: The socket server has closed.");
/*      */       }
/*      */     });
/*      */   }
/*      */   
/*      */   private synchronized void notifyServer() {
/*  238 */     notify();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public synchronized void close()
/*      */   {
/*  245 */     this.stop = true;
/*  246 */     interrupt();
/*  247 */     this.socketServer.close();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public synchronized Object threadMain()
/*      */     throws Exception
/*      */   {
/*  257 */     notifyTextReceivers("Poker server: Starting...");
/*  258 */     this.socketServer.start();
/*      */     try
/*      */     {
/*  261 */       notifyTextReceivers("Poker server: Awaiting players...");
/*  262 */       while (this.connectedPlayers.size() != this.numberOfPlayers)
/*      */       {
/*      */         try {
/*  265 */           wait();
/*      */         } catch (InterruptedException ie) {}
/*  267 */         if (this.stop)
/*      */         {
/*  269 */           notifyTextReceivers("Poker server: Shutting down before all players have joined.");
/*  270 */           return null;
/*      */         }
/*      */       }
/*  273 */       notifyTextReceivers("Poker server: All players connected.");
/*      */       
/*      */ 
/*      */ 
/*  277 */       int roundsPlayed = 0;
/*  278 */       this.startPlayerIndex = (this.activePlayers.size() - 1);
/*      */       
/*      */ 
/*  281 */       while (this.activePlayers.size() > 0)
/*      */       {
/*  283 */         roundsPlayed++;
/*  284 */         increasePlayedRoundsCount();
/*  285 */         notifyPlayersNewRound(roundsPlayed);
/*  286 */         notifyTextReceivers("Poker server: Round " + roundsPlayed + " started.");
/*      */         
/*  288 */         notifyTextReceivers("Poker server: Sending current standing.");
/*  289 */         notifyPlayersStandings();
/*  290 */         notifyGameObserversStandings();
/*      */         
/*  292 */         notifyTextReceivers("Poker server: Shuffling deck.");
/*  293 */         this.deckOfCards.shuffle();
/*      */         
/*  295 */         resetPlayersBet();
/*      */         
/*  297 */         this.maximumBet = 0;
/*      */         
/*  299 */         this.startPlayerIndex += 1;
/*  300 */         if ((this.startPlayerIndex >= this.activePlayers.size()) || (this.startPlayerIndex < 0)) {
/*  301 */           this.startPlayerIndex = 0;
/*      */         }
/*  303 */         notifyTextReceivers("Poker server: Dealing cards.");
/*  304 */         sendPlayersNewHands(this.deckOfCards);
/*  305 */         notifyGameObserversHands();
/*      */         
/*  307 */         if ((this.raisingAnte) && (roundsPlayed % this.raiseAnteAfterRoundsCount == 1))
/*      */         {
/*      */ 
/*  310 */           if (roundsPlayed > 1) {
/*  311 */             this.ante <<= 1;
/*      */           }
/*  313 */           this.socketServer.sendMessage("Ante_Changed " + this.ante);
/*      */         }
/*      */         
/*  316 */         forceBetAnte();
/*      */         
/*  318 */         this.minimumRaise = 1;
/*      */         
/*  320 */         doBettingRound(this.maximumBet);
/*      */         
/*      */ 
/*  323 */         if (this.timeToSleep > 0L) {
/*  324 */           Thread.sleep(this.timeToSleep);
/*      */         }
/*      */         
/*  327 */         doDraw();
/*  328 */         notifyGameObserversHands();
/*      */         
/*  330 */         this.minimumRaise = 1;
/*      */         
/*  332 */         doBettingRound(this.maximumBet);
/*      */         
/*  334 */         doShowdown();
/*      */         
/*  336 */         restoreNormalPlayerStatus();
/*      */         
/*  338 */         setBrokePlayersToNotActive();
/*      */         
/*  340 */         if (this.activePlayers.size() == 1)
/*      */         {
/*      */ 
/*      */ 
/*  344 */           ServerPlayerData player = getActivePlayer(0);
/*  345 */           this.activePlayers.removeElementAt(0);
/*  346 */           player.increasePlayedRounds();
/*  347 */           this.gameOverPlayers.addFirst(new PlayerResult(player.getName(), player.getPlayedRounds(), player.getRemainingChips()));
/*      */         }
/*      */         
/*      */ 
/*  351 */         if (this.timeToSleep > 0L) {
/*  352 */           Thread.sleep(this.timeToSleep);
/*      */         }
/*      */       }
/*      */       
/*  356 */       notifyPlayersFinalStandings();
/*      */       
/*      */ 
/*  359 */       sendInfoGameOver();
/*      */     }
/*      */     catch (Exception e) {
/*  362 */       notifyTextReceivers("Poker server: Exiting abruptly due to a thrown exception: " + e);
/*  363 */       throw e;
/*      */     }
/*      */     finally
/*      */     {
/*  367 */       if (this.timeToSleep > 0L) {
/*  368 */         Thread.sleep(this.timeToSleep);
/*      */       }
/*  370 */       this.socketServer.close();
/*      */     }
/*      */     
/*      */ 
/*  374 */     return getFinalResult();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void addNewPlayer(int clientID, String playerName)
/*      */   {
/*  384 */     ServerPlayerData player = new ServerPlayerData(clientID);
/*  385 */     player.setName(playerName);
/*  386 */     player.setRemainingChips(this.startChips);
/*  387 */     player.setChipsInPot(0);
/*  388 */     this.connectedPlayers.addElement(player);
/*  389 */     this.activePlayers.addElement(player);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private ServerPlayerData getConnectedPlayer(int connectedPlayerIndex)
/*      */   {
/*  399 */     return (ServerPlayerData)this.connectedPlayers.elementAt(connectedPlayerIndex);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private ServerPlayerData getActivePlayer(int activePlayerIndex)
/*      */   {
/*  408 */     return (ServerPlayerData)this.activePlayers.elementAt(activePlayerIndex);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private ServerPlayerData getPlayerForClientID(int clientID)
/*      */   {
/*  416 */     int startIndex = clientID >= this.connectedPlayers.size() ? 0 : clientID;
/*  417 */     int i = startIndex;
/*      */     for (;;)
/*      */     {
/*  420 */       ServerPlayerData player = (ServerPlayerData)this.connectedPlayers.elementAt(i);
/*  421 */       if (player.getClientID() == clientID)
/*  422 */         return player;
/*  423 */       i++;
/*  424 */       if (i >= this.connectedPlayers.size())
/*  425 */         i = 0;
/*  426 */       if (i == startIndex) {
/*  427 */         throw new IllegalArgumentException("No player has the client ID " + clientID + ".");
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private int getActiveStatusCount(int status)
/*      */   {
/*  437 */     int count = 0;
/*  438 */     for (IPlayerIterator players = getActivePlayerIterator(0); players.hasNext();)
/*      */     {
/*  440 */       if (players.next().getStatus() == status)
/*  441 */         count++;
/*      */     }
/*  443 */     return count;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int getActiveNotStatusCount(int status)
/*      */   {
/*  452 */     int count = 0;
/*  453 */     for (IPlayerIterator players = getActivePlayerIterator(0); players.hasNext();)
/*      */     {
/*  455 */       if (players.next().getStatus() != status)
/*  456 */         count++;
/*      */     }
/*  458 */     return count;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void increasePlayedRoundsCount()
/*      */   {
/*  465 */     for (int i = 0; i < this.activePlayers.size(); i++)
/*      */     {
/*  467 */       ServerPlayerData player = getActivePlayer(i);
/*  468 */       player.increasePlayedRounds();
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public IForwardIterator getFinalResult()
/*      */   {
/*  478 */     if (this.activePlayers.size() > 0) {
/*  479 */       throw new IllegalStateException("There is not yet a final result to return.");
/*      */     }
/*  481 */     return this.gameOverPlayers.iterator();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   public IForwardIterator getDisconnectedPlayers()
/*      */   {
/*  489 */     return this.disconnectedPlayers.iterator();
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public void addTextReceiver(ITextReceiver receiver)
/*      */   {
/*  498 */     this.textReceivers.addElement(receiver);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void removeTextReceiver(ITextReceiver receiver)
/*      */   {
/*  505 */     this.textReceivers.removeElement(receiver);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void notifyTextReceivers(String messageText)
/*      */   {
/*  512 */     Enumeration receivers = this.textReceivers.elements();
/*  513 */     while (receivers.hasMoreElements()) {
/*      */       try {
/*  515 */         ((ITextReceiver)receivers.nextElement()).textReceived(this, messageText);
/*      */       }
/*      */       catch (Exception e) {}
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void addGameObserver(IGameObserver observer)
/*      */   {
/*  525 */     this.gameObservers.addElement(observer);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public void removeGameObserver(IGameObserver observer)
/*      */   {
/*  532 */     this.gameObservers.removeElement(observer);
/*      */   }
/*      */   
/*      */ 
/*      */   public void notifyGameObserversStandings()
/*      */   {
/*  538 */     Enumeration observers = this.gameObservers.elements();
/*  539 */     while (observers.hasMoreElements())
/*      */     {
/*  541 */       IGameObserver currentObserver = (IGameObserver)observers.nextElement();
/*  542 */       IPlayerIterator players = getConnectedPlayerIterator();
/*  543 */       while (players.hasNext())
/*      */       {
/*  545 */         ServerPlayerData player = players.next();
/*      */         try {
/*  547 */           currentObserver.Standing(this, player.getName(), player.getRemainingChips());
/*      */         }
/*      */         catch (Exception e) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */   public void notifyGameObserversHands()
/*      */   {
/*  556 */     Enumeration observers = this.gameObservers.elements();
/*  557 */     while (observers.hasMoreElements())
/*      */     {
/*  559 */       IGameObserver currentObserver = (IGameObserver)observers.nextElement();
/*  560 */       IPlayerIterator players = getConnectedPlayerIterator();
/*  561 */       while (players.hasNext())
/*      */       {
/*  563 */         ServerPlayerData player = players.next();
/*      */         try {
/*  565 */           currentObserver.Hand(this, player.getName(), player.getStatus() != 3 ? player.getHand() : null);
/*      */         }
/*      */         catch (Exception e) {}
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void notifyPlayersNewRound(int round)
/*      */   {
/*  576 */     this.socketServer.sendMessage("Round " + round);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void notifyPlayersStandings()
/*      */   {
/*  584 */     for (int i = 0; i < this.connectedPlayers.size(); i++) {
/*  585 */       ServerPlayerData player = getConnectedPlayer(i);
/*  586 */       this.socketServer.sendMessage("Chips " + player.getName() + " " + player.getRemainingChips());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void resetPlayersBet()
/*      */   {
/*  596 */     for (int i = 0; i < this.activePlayers.size(); i++) {
/*  597 */       ServerPlayerData player = getActivePlayer(i);
/*  598 */       player.setChipsInPot(0);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void sendPlayersNewHands(Deck deckOfCards)
/*      */   {
/*  608 */     for (int i = 0; i < this.activePlayers.size(); i++) {
/*  609 */       ServerPlayerData player = getActivePlayer(i);
/*  610 */       player.getHand().makeEmpty();
/*      */     }
/*      */     
/*  613 */     for (int cardIndex = 0; cardIndex < 5; cardIndex++)
/*      */     {
/*  615 */       for (int playerIndexOffset = 0; playerIndexOffset < this.activePlayers.size(); playerIndexOffset++)
/*      */       {
/*  617 */         ServerPlayerData player = getActivePlayer((this.startPlayerIndex + playerIndexOffset) % this.activePlayers.size());
/*  618 */         player.getHand().addCard(deckOfCards.deal());
/*      */       }
/*      */     }
/*      */     
/*  622 */     for (int i = 0; i < this.activePlayers.size(); i++) {
/*  623 */       ServerPlayerData player = getActivePlayer(i);
/*      */       
/*  625 */       this.socketServer.sendMessage(player.getClientID(), "Cards" + player.getHand().toString());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void forceBetAnte()
/*      */   {
/*  636 */     notifyTextReceivers("Poker server: Forcing ante bets.");
/*      */     
/*  638 */     for (int playerIndexOffset = 0; playerIndexOffset < this.activePlayers.size(); playerIndexOffset++)
/*      */     {
/*  640 */       ServerPlayerData player = getActivePlayer((this.startPlayerIndex + playerIndexOffset) % this.activePlayers.size());
/*      */       
/*  642 */       if (player.getRemainingChips() > this.ante)
/*      */       {
/*  644 */         player.moveChipsToPot(this.ante);
/*      */         
/*  646 */         sendInfoPlayerForcedBet(player, player.getChipsInPot());
/*      */       }
/*      */       else
/*      */       {
/*  650 */         player.moveChipsToPot(player.getRemainingChips());
/*  651 */         setPlayerStatus(player, 2);
/*      */         
/*  653 */         sendInfoPlayerAllIn(player, player.getChipsInPot());
/*      */       }
/*      */       
/*  656 */       if (player.getChipsInPot() > this.maximumBet) {
/*  657 */         this.maximumBet = player.getChipsInPot();
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void doBettingRound(int maxBetAtBettingRoundStart)
/*      */   {
/*  666 */     notifyTextReceivers("Poker server: Betting round started.");
/*      */     
/*  668 */     if (getActiveStatusCount(0) <= 1)
/*      */     {
/*  670 */       notifyTextReceivers("Poker server: Betting round ended due to not enough players that can make additional bets.");
/*  671 */       return;
/*      */     }
/*      */     
/*  674 */     ServerPlayerData lastRaiserPlayer = null;
/*  675 */     IPlayerIterator playerIterator = getActivePlayerRingIterator(this.startPlayerIndex);
/*      */     for (;;)
/*      */     {
/*  678 */       notifyTextReceivers("Maximum bet is " + this.maximumBet + ", minimum raise is " + this.minimumRaise + " (to " + (this.maximumBet + this.minimumRaise) + ").");
/*      */       
/*  680 */       if (getActiveNotStatusCount(1) <= 1) {
/*      */         break;
/*      */       }
/*  683 */       if (getActiveStatusCount(0) == 0) {
/*      */         break;
/*      */       }
/*  686 */       ServerPlayerData player = playerIterator.next();
/*      */       
/*  688 */       if (player == lastRaiserPlayer) {
/*      */         break;
/*      */       }
/*  691 */       if ((player.getStatus() != 1) && (player.getStatus() != 2))
/*      */       {
/*      */ 
/*  694 */         if (!player.isConnected())
/*      */         {
/*      */ 
/*  697 */           setPlayerStatus(player, 1);
/*  698 */           sendInfoPlayerFold(player);
/*      */         }
/*      */         else
/*      */         {
/*  702 */           synchronized (this.lock)
/*      */           {
/*  704 */             if (this.maximumBet == maxBetAtBettingRoundStart)
/*      */             {
/*      */               try
/*      */               {
/*  708 */                 this.expectingAnswerFromClientID = player.getClientID();
/*  709 */                 this.receivedAnswerFromClientID = null;
/*  710 */                 sendQueryPlayerOpenAction(this.expectingAnswerFromClientID, this.maximumBet + this.minimumRaise, player.getChipsInPot(), player.getRemainingChips());
/*      */                 try {
/*  712 */                   this.lock.wait(this.maximumResponseTime);
/*      */                 } catch (InterruptedException ie) {}
/*  714 */                 this.expectingAnswerFromClientID = -1;
/*      */                 
/*  716 */                 if (this.receivedAnswerFromClientID == null)
/*  717 */                   throw new IOException("No response from player.");
/*  718 */                 if (this.receivedAnswerFromClientID[0].equals("Open"))
/*      */                 {
/*  720 */                   int openPlayerTotalAmount = Integer.parseInt(this.receivedAnswerFromClientID[1]);
/*  721 */                   if ((openPlayerTotalAmount < this.maximumBet + this.minimumRaise) || (openPlayerTotalAmount > player.getChipsInPot() + player.getRemainingChips()))
/*  722 */                     throw new IllegalStateException("Player cannot open to less than the specified raise; nor to more than the players total amount of chips.");
/*  723 */                   int raise = openPlayerTotalAmount - player.getChipsInPot();
/*  724 */                   this.maximumBet += raise;
/*  725 */                   this.minimumRaise = raise;
/*  726 */                   player.moveChipsToPot(raise);
/*  727 */                   lastRaiserPlayer = player;
/*      */                   
/*      */ 
/*  730 */                   if (player.getRemainingChips() > 0) {
/*  731 */                     sendInfoPlayerOpen(player, openPlayerTotalAmount);
/*      */                   }
/*      */                   else {
/*  734 */                     setPlayerStatus(player, 2);
/*  735 */                     sendInfoPlayerAllIn(player, openPlayerTotalAmount);
/*      */                   }
/*      */                 }
/*  738 */                 else if (this.receivedAnswerFromClientID[0].equals("All-in"))
/*      */                 {
/*  740 */                   int allInChipCount = player.getRemainingChips();
/*  741 */                   player.moveChipsToPot(allInChipCount);
/*      */                   
/*  743 */                   int raise = player.getChipsInPot() - this.maximumBet;
/*  744 */                   if (raise > this.minimumRaise)
/*      */                   {
/*  746 */                     this.minimumRaise = raise;
/*  747 */                     lastRaiserPlayer = player;
/*      */                   }
/*  749 */                   if (this.maximumBet < player.getChipsInPot()) {
/*  750 */                     this.maximumBet = player.getChipsInPot();
/*      */                   }
/*  752 */                   setPlayerStatus(player, 2);
/*      */                   
/*  754 */                   sendInfoPlayerAllIn(player, player.getChipsInPot());
/*      */                 }
/*  756 */                 else if (this.receivedAnswerFromClientID[0].equals("Check"))
/*      */                 {
/*  758 */                   sendInfoPlayerCheck(player);
/*      */                   
/*  760 */                   if (lastRaiserPlayer == null) {
/*  761 */                     lastRaiserPlayer = player;
/*      */                   }
/*      */                 } else {
/*  764 */                   throw new IllegalArgumentException("Expected opening action, received answer '" + this.receivedAnswerFromClientID[0] + "'.");
/*      */                 }
/*      */               }
/*      */               catch (Exception e) {
/*  768 */                 notifyTextReceivers("Poker server: Disconnecting player '" + player.getName() + "'. Error during betting round: '" + e + "'.");
/*  769 */                 setPlayerStatus(player, 1);
/*  770 */                 setPlayerDisconnected(player);
/*      */                 
/*  772 */                 sendInfoPlayerFold(player);
/*      */               }
/*      */               
/*      */             }
/*      */             else
/*      */               try
/*      */               {
/*  779 */                 this.expectingAnswerFromClientID = player.getClientID();
/*  780 */                 this.receivedAnswerFromClientID = null;
/*  781 */                 sendQueryPlayerCallRaiseAction(this.expectingAnswerFromClientID, this.maximumBet, this.maximumBet + this.minimumRaise, player.getChipsInPot(), player.getRemainingChips());
/*      */                 try {
/*  783 */                   this.lock.wait(this.maximumResponseTime);
/*      */                 } catch (InterruptedException ie) {}
/*  785 */                 this.expectingAnswerFromClientID = -1;
/*      */                 
/*  787 */                 if (this.receivedAnswerFromClientID == null) {
/*  788 */                   throw new IOException("No response from player.");
/*      */                 }
/*  790 */                 if (this.receivedAnswerFromClientID[0].equals("Call"))
/*      */                 {
/*  792 */                   if (this.maximumBet > player.getChipsInPot() + player.getRemainingChips())
/*  793 */                     throw new IllegalStateException("Player cannot call; has to few chips.");
/*  794 */                   player.moveChipsToPot(this.maximumBet - player.getChipsInPot());
/*      */                   
/*  796 */                   sendInfoPlayerCall(player);
/*      */                 }
/*  798 */                 else if (this.receivedAnswerFromClientID[0].equals("Raise"))
/*      */                 {
/*  800 */                   int amountRaisedTo = Integer.parseInt(this.receivedAnswerFromClientID[1]);
/*  801 */                   if (amountRaisedTo > player.getChipsInPot() + player.getRemainingChips())
/*  802 */                     throw new IllegalStateException("Player cannot raise to " + amountRaisedTo + "; has to few chips.");
/*  803 */                   if (amountRaisedTo < this.maximumBet + this.minimumRaise)
/*  804 */                     throw new IllegalArgumentException("Player cannot raise to " + amountRaisedTo + "; must raise with at least " + this.minimumRaise + ".");
/*  805 */                   player.moveChipsToPot(amountRaisedTo - player.getChipsInPot());
/*  806 */                   this.minimumRaise = (amountRaisedTo - this.maximumBet);
/*  807 */                   this.maximumBet = amountRaisedTo;
/*  808 */                   lastRaiserPlayer = player;
/*      */                   
/*      */ 
/*  811 */                   if (player.getRemainingChips() > 0) {
/*  812 */                     sendInfoPlayerRaise(player, amountRaisedTo);
/*      */                   }
/*      */                   else {
/*  815 */                     setPlayerStatus(player, 2);
/*  816 */                     sendInfoPlayerAllIn(player, amountRaisedTo);
/*      */                   }
/*      */                 }
/*  819 */                 else if (this.receivedAnswerFromClientID[0].equals("All-in"))
/*      */                 {
/*  821 */                   int allInChipCount = player.getRemainingChips();
/*  822 */                   player.moveChipsToPot(allInChipCount);
/*      */                   
/*  824 */                   int raise = player.getChipsInPot() - this.maximumBet;
/*  825 */                   if (raise > this.minimumRaise)
/*      */                   {
/*  827 */                     this.minimumRaise = raise;
/*  828 */                     lastRaiserPlayer = player;
/*      */                   }
/*  830 */                   if (this.maximumBet < player.getChipsInPot()) {
/*  831 */                     this.maximumBet = player.getChipsInPot();
/*      */                   }
/*  833 */                   setPlayerStatus(player, 2);
/*      */                   
/*  835 */                   sendInfoPlayerAllIn(player, player.getChipsInPot());
/*      */                 }
/*  837 */                 else if (this.receivedAnswerFromClientID[0].equals("Fold"))
/*      */                 {
/*  839 */                   setPlayerStatus(player, 1);
/*  840 */                   sendInfoPlayerFold(player);
/*      */                 }
/*      */                 else {
/*  843 */                   throw new IllegalArgumentException("Expected raising action, received answer '" + this.receivedAnswerFromClientID[0] + "'.");
/*      */                 }
/*      */               }
/*      */               catch (Exception e) {
/*  847 */                 notifyTextReceivers("Poker server: Disconnecting player '" + player.getName() + "'. Error during betting round: " + e + ".");
/*  848 */                 setPlayerDisconnected(player);
/*  849 */                 setPlayerStatus(player, 1);
/*      */                 
/*  851 */                 sendInfoPlayerFold(player);
/*      */               }
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*  857 */     notifyTextReceivers("Poker server: Betting round ended.");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void doDraw()
/*      */   {
/*  866 */     notifyTextReceivers("Poker server: Draw started...");
/*      */     
/*  868 */     int activeNotFoldedCount = getActiveNotStatusCount(1);
/*  869 */     if (activeNotFoldedCount > 1)
/*      */     {
/*  871 */       for (int playerIndexOffset = 0; playerIndexOffset < this.activePlayers.size(); playerIndexOffset++)
/*      */       {
/*  873 */         ServerPlayerData player = getActivePlayer((this.startPlayerIndex + playerIndexOffset) % this.activePlayers.size());
/*      */         
/*  875 */         int cardsExchanged = 0;
/*      */         
/*  877 */         if (player.isConnected())
/*      */         {
/*  879 */           synchronized (this.lock)
/*      */           {
/*      */             try
/*      */             {
/*  883 */               this.expectingAnswerFromClientID = player.getClientID();
/*  884 */               this.receivedAnswerFromClientID = null;
/*  885 */               this.socketServer.sendMessage(this.expectingAnswerFromClientID, "Draw?");
/*      */               try {
/*  887 */                 this.lock.wait(this.maximumResponseTime);
/*      */               } catch (InterruptedException ie) {}
/*  889 */               this.expectingAnswerFromClientID = -1;
/*      */               
/*  891 */               if (this.receivedAnswerFromClientID == null)
/*  892 */                 throw new IOException("No response from player.");
/*  893 */               if (!this.receivedAnswerFromClientID[0].equals("Throws")) {
/*  894 */                 throw new IllegalArgumentException("Expected answer 'Throws', received answer '" + this.receivedAnswerFromClientID[0] + "'.");
/*      */               }
/*  896 */               Hand hand = player.getHand();
/*  897 */               for (int i = 1; i < this.receivedAnswerFromClientID.length; i++)
/*      */               {
/*  899 */                 Card thrownCard = new Card(this.receivedAnswerFromClientID[i]);
/*  900 */                 int indexOfCard = getIndexOfCard(hand, thrownCard);
/*  901 */                 if (indexOfCard < 0)
/*  902 */                   throw new IllegalArgumentException("The card '" + thrownCard + "' cannot be thrown; it is not in the hand '" + hand + "'.");
/*  903 */                 hand.setCard(indexOfCard, this.deckOfCards.deal());
/*  904 */                 cardsExchanged++;
/*      */               }
/*      */               
/*  907 */               if (this.receivedAnswerFromClientID.length > 1)
/*      */               {
/*      */ 
/*  910 */                 this.socketServer.sendMessage(player.getClientID(), "Cards" + player.getHand().toString());
/*      */               }
/*      */             }
/*      */             catch (Exception e)
/*      */             {
/*  915 */               notifyTextReceivers("Poker server: Disconnecting player '" + player.getName() + "'. Error during draw: " + e + ".");
/*  916 */               setPlayerDisconnected(player);
/*  917 */               activeNotFoldedCount--; if (activeNotFoldedCount < 2) {
/*      */                 break;
/*      */               }
/*      */             }
/*      */           }
/*      */         }
/*  923 */         sendInfoPlayerDraw(player, cardsExchanged);
/*      */       }
/*      */     }
/*  926 */     notifyTextReceivers("Poker server: Draw ended.");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void doShowdown()
/*      */   {
/*  934 */     notifyTextReceivers("Poker server: Showdown started...");
/*      */     
/*  936 */     int totalPotValue = 0;
/*  937 */     int notFoldedCount = 0;
/*  938 */     OrderedCountingSet playersBet = new OrderedCountingSet(se.lypson.common.StandardComparators.getIntegerComparator());
/*  939 */     for (int i = 0; i < this.activePlayers.size(); i++)
/*      */     {
/*  941 */       ServerPlayerData player = getActivePlayer(i);
/*      */       
/*  943 */       notifyTextReceivers("Player " + player.getName() + ": Status is " + player.getStatus() + ", chips in pot is " + player.getChipsInPot() + ".");
/*      */       
/*  945 */       totalPotValue += player.getChipsInPot();
/*  946 */       playersBet.add(new Integer(player.getChipsInPot()));
/*  947 */       if (player.getStatus() != 1)
/*      */       {
/*  949 */         notFoldedCount++;
/*  950 */         player.calculateRank();
/*      */       }
/*      */       else {
/*  953 */         player.resetRank();
/*      */       } }
/*  955 */     if (notFoldedCount == 1)
/*      */     {
/*      */ 
/*  958 */       for (int i = 0; i < this.activePlayers.size(); i++)
/*      */       {
/*  960 */         ServerPlayerData player = getActivePlayer(i);
/*  961 */         if (player.getStatus() != 1)
/*      */         {
/*  963 */           player.setRemainingChips(player.getRemainingChips() + totalPotValue);
/*  964 */           sendInfoRoundUndisputedWin(player, totalPotValue);
/*  965 */           notifyTextReceivers("Poker server: Player " + player.getName() + " won " + totalPotValue + " chips undisputed.");
/*  966 */           break;
/*      */         }
/*      */         
/*      */       }
/*      */     }
/*      */     else
/*      */     {
/*  973 */       int previousPotInput = 0;
/*  974 */       Enumeration playerPotInputs = playersBet.enumeration();
/*  975 */       while (playerPotInputs.hasMoreElements())
/*      */       {
/*  977 */         int thisPotInput = ((Integer)playerPotInputs.nextElement()).intValue();
/*  978 */         int raise = thisPotInput - previousPotInput;
/*  979 */         PotData currentPot = new PotData(raise);
/*      */         
/*  981 */         for (int i = 0; i < this.activePlayers.size(); i++)
/*      */         {
/*  983 */           ServerPlayerData player = getActivePlayer(i);
/*      */           
/*  985 */           if (player.getChipsInPot() >= raise)
/*      */           {
/*  987 */             currentPot.players.addElement(player);
/*  988 */             player.setChipsInPot(player.getChipsInPot() - raise);
/*      */           }
/*      */         }
/*  991 */         currentPot.orderPlayersByHands();
/*  992 */         int numberOfPotWinners = currentPot.getNumberOfWinners();
/*  993 */         int chipsWonPerPlayer = currentPot.getPotSize() / numberOfPotWinners;
/*      */         
/*      */ 
/*  996 */         int roundOffChips = currentPot.getPotSize() - chipsWonPerPlayer * numberOfPotWinners;
/*  997 */         for (int i = 0; i < numberOfPotWinners; i++)
/*      */         {
/*  999 */           ServerPlayerData player = (ServerPlayerData)currentPot.players.elementAt(i);
/* 1000 */           player.increaseChipsWonCurrentRound(chipsWonPerPlayer);
/* 1001 */           if (roundOffChips > 0)
/*      */           {
/* 1003 */             player.increaseChipsWonCurrentRound(1);
/* 1004 */             roundOffChips--;
/*      */           }
/*      */         }
/* 1007 */         notifyTextReceivers("Poker server: Pot data: " + currentPot.players.size() + " player(s), " + raise + " chips/player, " + numberOfPotWinners + " winner(s).");
/*      */         
/* 1009 */         previousPotInput = thisPotInput;
/*      */       }
/*      */       
/*      */ 
/* 1013 */       for (int i = 0; i < this.activePlayers.size(); i++)
/*      */       {
/* 1015 */         ServerPlayerData player = getActivePlayer(i);
/* 1016 */         if (player.getStatus() != 1) {
/* 1017 */           sendInfoPlayerHand(player, player.getHand());
/*      */         }
/*      */       }
/*      */       
/* 1021 */       for (int i = 0; i < this.activePlayers.size(); i++)
/*      */       {
/* 1023 */         ServerPlayerData player = getActivePlayer(i);
/* 1024 */         if (player.getChipsWonCurrentRound() > 0)
/*      */         {
/* 1026 */           sendInfoRoundResult(player, player.getChipsWonCurrentRound());
/* 1027 */           notifyTextReceivers("Poker server: Player " + player.getName() + " won " + player.getChipsWonCurrentRound() + " chips.");
/*      */           
/* 1029 */           player.moveChipsWonToRemainingChips();
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/* 1035 */     for (int i = 0; i < this.activePlayers.size(); i++)
/*      */     {
/* 1037 */       ServerPlayerData player = getActivePlayer(i);
/* 1038 */       player.resetRank();
/* 1039 */       player.setChipsInPot(0);
/*      */     }
/*      */     
/* 1042 */     notifyTextReceivers("Poker server: Showdown ended.");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private int getIndexOfCard(Hand hand, Card card)
/*      */   {
/* 1053 */     for (int i = 1; i <= hand.size(); i++)
/* 1054 */       if (hand.getCard(i).getIndex() == card.getIndex())
/* 1055 */         return i;
/* 1056 */     return -1;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void restoreNormalPlayerStatus()
/*      */   {
/* 1063 */     for (int i = 0; i < this.activePlayers.size(); i++) {
/* 1064 */       ServerPlayerData player = getActivePlayer(i);
/* 1065 */       if ((player.getStatus() == 1) || (player.getStatus() == 2)) {
/* 1066 */         setPlayerStatus(player, 0);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void setBrokePlayersToNotActive()
/*      */   {
/* 1078 */     int i = 0;
/* 1079 */     while (i < this.activePlayers.size())
/*      */     {
/* 1081 */       ServerPlayerData player = getActivePlayer(i);
/*      */       
/* 1083 */       if ((player.getStatus() != 3) && (player.getRemainingChips() <= 0))
/*      */       {
/* 1085 */         setPlayerStatus(player, 3);
/* 1086 */         this.gameOverPlayers.addFirst(new PlayerResult(player.getName(), player.getPlayedRounds(), player.getRemainingChips()));
/* 1087 */         this.activePlayers.removeElementAt(i);
/* 1088 */         if (i <= this.startPlayerIndex) {
/* 1089 */           this.startPlayerIndex -= 1;
/*      */         }
/*      */       } else {
/* 1092 */         i++;
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void notifyPlayersFinalStandings()
/*      */   {
/* 1101 */     for (int i = 0; i < this.connectedPlayers.size(); i++) {
/* 1102 */       ServerPlayerData player = getConnectedPlayer(i);
/* 1103 */       this.socketServer.sendMessage("Result " + player.getName() + " " + player.getPlayedRounds() + " " + player.getRemainingChips());
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   private void setPlayerDisconnected(ServerPlayerData player)
/*      */   {
/* 1111 */     player.setConnected(false);
/* 1112 */     this.disconnectedPlayers.addLast(player);
/* 1113 */     notifyTextReceivers("Player " + player.getName() + " disconnected.");
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   private void setPlayerStatus(ServerPlayerData player, int status)
/*      */   {
/* 1121 */     player.setStatus(status);
/* 1122 */     notifyTextReceivers("Player " + player.getName() + ": State changed to " + player.getStatus());
/*      */   }
/*      */   
/*      */   private void sendQueryPlayerOpenAction(int clientID, int minimumPotAfterOpen, int playersCurrentBet, int playersRemainingChips) {
/* 1126 */     this.socketServer.sendMessage(clientID, "Open? " + minimumPotAfterOpen + " " + playersCurrentBet + " " + playersRemainingChips);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void sendQueryPlayerCallRaiseAction(int clientID, int maximumBet, int minimumAmountToRaiseTo, int playersCurrentBet, int playersRemainingChips)
/*      */   {
/* 1141 */     this.socketServer.sendMessage(clientID, "Call/Raise? " + maximumBet + " " + minimumAmountToRaiseTo + " " + playersCurrentBet + " " + playersRemainingChips);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private void sendInfoGameOver()
/*      */   {
/* 1153 */     this.socketServer.sendMessage("Game_Over");
/*      */   }
/*      */   
/* 1156 */   private void sendInfoPlayerForcedBet(ServerPlayerData player, int chipCount) { this.socketServer.sendMessage("Forced_Bet " + player.getName() + " " + chipCount); }
/*      */   
/*      */   private void sendInfoPlayerOpen(ServerPlayerData player, int chipCount)
/*      */   {
/* 1160 */     this.socketServer.sendMessage("Player_Open " + player.getName() + " " + chipCount);
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerCheck(ServerPlayerData player) {
/* 1164 */     this.socketServer.sendMessage("Player_Check " + player.getName() + " ");
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerRaise(ServerPlayerData player, int chipCount) {
/* 1168 */     this.socketServer.sendMessage("Player_Raise " + player.getName() + " " + chipCount);
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerCall(ServerPlayerData player) {
/* 1172 */     this.socketServer.sendMessage("Player_Call " + player.getName() + " ");
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerFold(ServerPlayerData player) {
/* 1176 */     this.socketServer.sendMessage("Player_Fold " + player.getName() + " ");
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerAllIn(ServerPlayerData player, int chipCount) {
/* 1180 */     this.socketServer.sendMessage("Player_All-in " + player.getName() + " " + chipCount);
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerDraw(ServerPlayerData player, int cardsExchangedCount) {
/* 1184 */     this.socketServer.sendMessage("Player_Draw " + player.getName() + " " + cardsExchangedCount);
/*      */   }
/*      */   
/*      */   private void sendInfoPlayerHand(ServerPlayerData player, Hand hand) {
/* 1188 */     this.socketServer.sendMessage("Player_Hand " + player.getName() + hand);
/*      */   }
/*      */   
/*      */   private void sendInfoRoundUndisputedWin(ServerPlayerData player, int winAmount) {
/* 1192 */     this.socketServer.sendMessage("Round_Win_Undisputed " + player.getName() + " " + winAmount);
/*      */   }
/*      */   
/*      */   private void sendInfoRoundResult(ServerPlayerData player, int winAmount) {
/* 1196 */     this.socketServer.sendMessage("Round_result " + player.getName() + " " + winAmount);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IPlayerIterator getConnectedPlayerIterator()
/*      */   {
/* 1205 */     new IPlayerIterator() {
/* 1206 */       int currentIndex = 0;
/*      */       
/*      */       public boolean hasNext() {
/* 1209 */         return this.currentIndex < PokerServer.this.connectedPlayers.size();
/*      */       }
/*      */       
/*      */       public ServerPlayerData next() {
/* 1213 */         return (ServerPlayerData)PokerServer.this.connectedPlayers.elementAt(this.currentIndex++);
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IPlayerIterator getActivePlayerIterator(int iterationStartIndex)
/*      */   {
/* 1224 */     final int startIndex = iterationStartIndex;
/* 1225 */     new IPlayerIterator() {
/* 1226 */       int currentIndex = startIndex;
/* 1227 */       int remainingIterationSteps = PokerServer.this.activePlayers.size();
/*      */       
/*      */       public boolean hasNext() {
/* 1230 */         return this.remainingIterationSteps > 0;
/*      */       }
/*      */       
/*      */       public ServerPlayerData next() {
/* 1234 */         this.remainingIterationSteps -= 1;
/* 1235 */         ServerPlayerData nextPlayer = (ServerPlayerData)PokerServer.this.activePlayers.elementAt(this.currentIndex);
/* 1236 */         if (++this.currentIndex >= PokerServer.this.activePlayers.size())
/* 1237 */           this.currentIndex = 0;
/* 1238 */         return nextPlayer;
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IPlayerIterator getActivePlayerRingIterator(int iterationStartIndex)
/*      */   {
/* 1251 */     final int startIndex = iterationStartIndex;
/* 1252 */     new IPlayerIterator() {
/* 1253 */       int currentIndex = startIndex - 1;
/*      */       
/*      */       public boolean hasNext() {
/* 1256 */         return PokerServer.this.activePlayers.size() > 0;
/*      */       }
/*      */       
/*      */       public ServerPlayerData next() {
/* 1260 */         return (ServerPlayerData)PokerServer.this.activePlayers.elementAt(this.currentIndex = this.currentIndex + 1 < PokerServer.this.activePlayers.size() ? this.currentIndex + 1 : 0);
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   private IComparator getPlayerOrderByHandComparator()
/*      */   {
/* 1271 */     new IComparator() {
/*      */       public int compare(Object first, Object second) {
/* 1273 */         ServerPlayerData firstPlayer = (ServerPlayerData)first;
/* 1274 */         ServerPlayerData secondPlayer = (ServerPlayerData)second;
/*      */         
/* 1276 */         return secondPlayer.getRank() - firstPlayer.getRank();
/*      */       }
/*      */     };
/*      */   }
/*      */   
/*      */   private static abstract interface IPlayerIterator
/*      */   {
/*      */     public abstract boolean hasNext();
/*      */     
/*      */     public abstract ServerPlayerData next();
/*      */   }
/*      */ }


/* Location:              /Users/Dawidejdeholm/Desktop/Skola/AI/PokerProject/PokerServerGUI/PokerServerGui.jar!/com/medicwave/cardgame/poker/PokerServer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */