/*     */ package com.medicwave.cardgame.poker;
/*     */ 
/*     */ import ca.ualberta.cs.poker.Card;
/*     */ import ca.ualberta.cs.poker.Hand;
/*     */ import java.awt.Color;
/*     */ import java.awt.Dimension;
/*     */ import java.awt.Font;
/*     */ import java.awt.Graphics;
/*     */ import java.awt.Image;
/*     */ import java.util.Vector;
/*     */ import javax.swing.GroupLayout;
/*     */ import javax.swing.ImageIcon;
/*     */ import javax.swing.JPanel;
/*     */ import org.jdesktop.application.Application;
/*     */ import org.jdesktop.application.ResourceMap;
/*     */ 
/*     */ public class PokerTablePanel extends JPanel
/*     */ {
/*     */   private static final Image BACKGROUND_IMAGE;
/*     */   private static final int COIN_OVERLAP_OFFSET = 4;
/*     */   
/*     */   private class PlayerStandingData
/*     */   {
/*     */     String playerName;
/*     */     int chipCount;
/*     */     Hand hand;
/*     */     
/*     */     public PlayerStandingData(String playerName, int chipCount, Hand hand)
/*     */     {
/*  30 */       this.playerName = playerName;
/*  31 */       this.chipCount = chipCount;
/*  32 */       this.hand = hand;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*  39 */   private static final Image[] COIN_IMAGES = new Image[6];
/*  40 */   private static final int[] COIN_VALUES = { 1, 5, 10, 25, 50, 100 };
/*  41 */   private static final int[] COIN_GROUP_X_OFFSET = { 18, 564, 18, 564, 292 };
/*  42 */   private static final int[] COIN_GROUP_Y_OFFSET = { 105, 105, 329, 329, 223 };
/*  43 */   private static final int[] COIN_STACK_X_OFFSET = { 41, 32, 25, 16, 9, 0 };
/*  44 */   private static final int[] COIN_STACK_Y_OFFSET = { 10, 0, 14, -4, 10, 0 };
/*  45 */   private static final int[] COIN_STACK_PAINTORDER = { 5, 3, 1, 4, 2, 0 };
/*     */   
/*  47 */   private static final Image[] CARDS_SPADES_IMAGES = new Image[13];
/*  48 */   private static final Image[] CARDS_HEARTS_IMAGES = new Image[13];
/*  49 */   private static final Image[] CARDS_CLUBS_IMAGES = new Image[13];
/*  50 */   private static final Image[] CARDS_DIAMONDS_IMAGES = new Image[13];
/*  51 */   private static final int[][][] CARD_POSITIONS = { { { 31, 156 }, { 85, 136 }, { 139, 156 }, { 193, 136 }, { 247, 156 } }, { { 355, 156 }, { 409, 136 }, { 463, 156 }, { 517, 136 }, { 571, 156 } }, { { 31, 380 }, { 85, 400 }, { 139, 380 }, { 193, 400 }, { 247, 380 } }, { { 355, 380 }, { 409, 400 }, { 463, 380 }, { 517, 400 }, { 571, 380 } }, { { 194, 268 }, { 248, 268 }, { 302, 268 }, { 356, 268 }, { 410, 268 } } };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  57 */   private static final Color COLOR_NORMAL = Color.YELLOW;
/*  58 */   private static final Color COLOR_BROKE = Color.RED;
/*  59 */   private static final int[] NAME_X_OFFSET = { 85, 463, 85, 463, 463 };
/*  60 */   private static final int[] NAME_Y_OFFSET = { 120, 120, 369, 369, 288 };
/*     */   
/*  62 */   private Vector<PlayerStandingData> playerStandings = new Vector(5);
/*  63 */   private final Object lock = new Object();
/*     */   
/*     */ 
/*     */ 
/*     */   static
/*     */   {
/*  69 */     ResourceMap resourceMap = ((PokerServerGuiApp)Application.getInstance(PokerServerGuiApp.class)).getContext().getResourceMap(PokerTablePanel.class);
/*     */     
/*     */ 
/*  72 */     BACKGROUND_IMAGE = resourceMap.getImageIcon("backgroundImage.imageIcon").getImage();
/*     */     
/*     */ 
/*  75 */     for (int i = 0; i < COIN_IMAGES.length; i++) {
/*  76 */       COIN_IMAGES[i] = resourceMap.getImageIcon("coins.imageIcon[" + i + "]").getImage();
/*     */     }
/*     */     
/*     */ 
/*  80 */     for (int i = 0; i < CARDS_SPADES_IMAGES.length; i++) {
/*  81 */       CARDS_SPADES_IMAGES[i] = resourceMap.getImageIcon("spades.imageIcon[" + i + "]").getImage();
/*     */     }
/*     */     
/*  84 */     for (int i = 0; i < CARDS_HEARTS_IMAGES.length; i++) {
/*  85 */       CARDS_HEARTS_IMAGES[i] = resourceMap.getImageIcon("hearts.imageIcon[" + i + "]").getImage();
/*     */     }
/*     */     
/*  88 */     for (int i = 0; i < CARDS_CLUBS_IMAGES.length; i++) {
/*  89 */       CARDS_CLUBS_IMAGES[i] = resourceMap.getImageIcon("clubs.imageIcon[" + i + "]").getImage();
/*     */     }
/*     */     
/*  92 */     for (int i = 0; i < CARDS_DIAMONDS_IMAGES.length; i++) {
/*  93 */       CARDS_DIAMONDS_IMAGES[i] = resourceMap.getImageIcon("diamonds.imageIcon[" + i + "]").getImage();
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public PokerTablePanel()
/*     */   {
/* 100 */     initComponents();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void initComponents()
/*     */   {
/* 123 */     ResourceMap resourceMap = ((PokerServerGuiApp)Application.getInstance(PokerServerGuiApp.class)).getContext().getResourceMap(PokerTablePanel.class);
/* 124 */     setBackground(resourceMap.getColor("Form.background"));
/* 125 */     setMaximumSize(new Dimension(640, 480));
/* 126 */     setMinimumSize(new Dimension(640, 480));
/* 127 */     setName("Form");
/* 128 */     setPreferredSize(new Dimension(640, 480));
/*     */     
/* 130 */     GroupLayout layout = new GroupLayout(this);
/* 131 */     setLayout(layout);
/* 132 */     layout.setHorizontalGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 640, 32767));
/*     */     
/*     */ 
/*     */ 
/* 136 */     layout.setVerticalGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 480, 32767));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void paintComponent(Graphics g)
/*     */   {
/* 148 */     super.paintComponent(g);
/* 149 */     drawDeck(g);
/* 150 */     drawGame(g);
/*     */   }
/*     */   
/*     */   public void setPlayerStanding(String playerName, int chipCount) {
/* 154 */     synchronized (this.lock)
/*     */     {
/* 156 */       boolean playerFound = false;
/* 157 */       for (int i = 0; i < this.playerStandings.size(); i++)
/*     */       {
/* 159 */         PlayerStandingData currentData = (PlayerStandingData)this.playerStandings.elementAt(i);
/* 160 */         if (currentData.playerName.equals(playerName))
/*     */         {
/* 162 */           currentData.chipCount = chipCount;
/* 163 */           playerFound = true;
/* 164 */           break;
/*     */         }
/*     */       }
/* 167 */       if (!playerFound)
/*     */       {
/*     */ 
/* 170 */         this.playerStandings.addElement(new PlayerStandingData(playerName, chipCount, null));
/*     */       }
/*     */     }
/* 173 */     repaint();
/*     */   }
/*     */   
/*     */   public void setPlayerHand(String playerName, Hand hand) {
/* 177 */     synchronized (this.lock)
/*     */     {
/* 179 */       boolean playerFound = false;
/* 180 */       for (int i = 0; i < this.playerStandings.size(); i++)
/*     */       {
/* 182 */         PlayerStandingData currentData = (PlayerStandingData)this.playerStandings.elementAt(i);
/* 183 */         if (currentData.playerName.equals(playerName))
/*     */         {
/* 185 */           currentData.hand = hand;
/* 186 */           playerFound = true;
/* 187 */           break;
/*     */         }
/*     */       }
/* 190 */       if (!playerFound)
/*     */       {
/*     */ 
/* 193 */         this.playerStandings.addElement(new PlayerStandingData(playerName, 0, hand));
/*     */       }
/*     */     }
/* 196 */     repaint();
/*     */   }
/*     */   
/*     */   public void clearPlayers() {
/* 200 */     this.playerStandings.clear();
/* 201 */     revalidate();
/*     */   }
/*     */   
/*     */   private void drawDeck(Graphics g) {
/* 205 */     int margin = 10;
/* 206 */     int x = 10;
/* 207 */     int y = 10;
/*     */     
/* 209 */     Image[][] deck = { CARDS_SPADES_IMAGES, CARDS_HEARTS_IMAGES, CARDS_CLUBS_IMAGES, CARDS_DIAMONDS_IMAGES };
/*     */     
/*     */     for (;;)
/*     */     {
/* 213 */       for (int suit = 0; suit < deck.length; suit++)
/*     */       {
/* 215 */         for (int i = 0; i < deck[suit].length; i++) {
/* 216 */           if (x + deck[suit][i].getWidth(null) >= getWidth())
/*     */           {
/* 218 */             x = 10;
/* 219 */             y += deck[suit][i].getHeight(null) + 10;
/*     */           }
/* 221 */           if (y + deck[suit][i].getHeight(null) >= getHeight())
/*     */             return;
/* 223 */           g.drawImage(deck[suit][i], x, y, null);
/* 224 */           x += deck[suit][i].getWidth(null) + 10;
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private void drawTableBackground(Graphics g)
/*     */   {
/* 235 */     g.drawImage(BACKGROUND_IMAGE, 0, 0, Color.GREEN, null);
/*     */   }
/*     */   
/*     */   private void drawGame(Graphics g)
/*     */   {
/* 240 */     Graphics tableGraphics = null;
/* 241 */     Font nameFont = null;
/*     */     try
/*     */     {
/* 244 */       tableGraphics = g.create((getWidth() - BACKGROUND_IMAGE.getWidth(null)) / 2, (getHeight() - BACKGROUND_IMAGE.getHeight(null)) / 2, BACKGROUND_IMAGE.getWidth(null), BACKGROUND_IMAGE.getHeight(null));
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 251 */       drawTableBackground(tableGraphics);
/*     */       
/* 253 */       nameFont = g.getFont().deriveFont(g.getFont().getSize() * 2.0F);
/* 254 */       tableGraphics.setFont(nameFont);
/*     */       
/* 256 */       synchronized (this.lock)
/*     */       {
/* 258 */         for (int i = 0; i < this.playerStandings.size(); i++)
/*     */         {
/* 260 */           PlayerStandingData currentData = (PlayerStandingData)this.playerStandings.elementAt(i);
/*     */           
/*     */ 
/* 263 */           drawCoinStack(tableGraphics, i, currentData.chipCount);
/*     */           
/* 265 */           drawCards(tableGraphics, i, currentData.hand);
/*     */           
/* 267 */           tableGraphics.setColor(currentData.chipCount > 0 ? COLOR_NORMAL : COLOR_BROKE);
/* 268 */           tableGraphics.drawString(currentData.playerName, NAME_X_OFFSET[i], NAME_Y_OFFSET[i]);
/*     */         }
/*     */       }
/*     */     }
/*     */     finally
/*     */     {
/* 274 */       if (tableGraphics != null)
/* 275 */         tableGraphics.dispose();
/*     */     }
/*     */   }
/*     */   
/*     */   private void drawCoinStack(Graphics g, int playerPositionIndex, int totalChipCount) {
/* 280 */     int GROUP_X = COIN_GROUP_X_OFFSET[playerPositionIndex];
/* 281 */     int GROUP_Y = COIN_GROUP_Y_OFFSET[playerPositionIndex];
/*     */     
/* 283 */     int[] numberOfCoins = new int[COIN_VALUES.length];
/*     */     
/*     */ 
/* 286 */     for (int coinIndex = COIN_VALUES.length - 1; coinIndex >= 0; coinIndex--)
/*     */     {
/* 288 */       numberOfCoins[coinIndex] = (totalChipCount / COIN_VALUES[coinIndex]);
/* 289 */       totalChipCount -= numberOfCoins[coinIndex] * COIN_VALUES[coinIndex];
/*     */     }
/*     */     
/* 292 */     for (int paintIndex = 0; paintIndex < COIN_STACK_PAINTORDER.length; paintIndex++)
/*     */     {
/* 294 */       int coinIndex = COIN_STACK_PAINTORDER[paintIndex];
/* 295 */       int x = GROUP_X + COIN_STACK_X_OFFSET[coinIndex];
/* 296 */       int y = GROUP_Y + COIN_STACK_Y_OFFSET[coinIndex];
/*     */       
/* 298 */       for (int stackIndex = 0; stackIndex < numberOfCoins[coinIndex]; y -= 4)
/*     */       {
/* 300 */         g.drawImage(COIN_IMAGES[coinIndex], x, y, null);stackIndex++;
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private void drawCards(Graphics g, int playerPositionIndex, Hand hand) {
/* 306 */     if (hand == null) {
/* 307 */       return;
/*     */     }
/* 309 */     int[][] cardPositions = CARD_POSITIONS[playerPositionIndex];
/*     */     
/* 311 */     for (int i = 0; (i < hand.size()) && (i < cardPositions.length); i++)
/*     */     {
/*     */ 
/* 314 */       Card currentCard = hand.getCard(i + 1);
/* 315 */       int suit = currentCard.getSuit();
/* 316 */       Image[] suitImages = suit == 1 ? CARDS_DIAMONDS_IMAGES : suit == 0 ? CARDS_CLUBS_IMAGES : suit == 2 ? CARDS_HEARTS_IMAGES : suit == 3 ? CARDS_SPADES_IMAGES : null;
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 321 */       int rank = currentCard.getRank();
/*     */       
/*     */ 
/* 324 */       int imageIndex = rank != 12 ? rank + 1 : 0;
/*     */       
/* 326 */       g.drawImage(suitImages[imageIndex], cardPositions[i][0], cardPositions[i][1], null);
/*     */     }
/*     */   }
/*     */ }


/* Location:              /Users/Dawidejdeholm/Desktop/Skola/AI/PokerProject/PokerServerGUI/PokerServerGui.jar!/com/medicwave/cardgame/poker/PokerTablePanel.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */