/*     */ package com.medicwave.socketlibrary;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.io.OutputStream;
/*     */ import java.io.PrintStream;
/*     */ import java.net.HttpURLConnection;
/*     */ import java.net.ServerSocket;
/*     */ import java.net.Socket;
/*     */ import java.net.URL;
/*     */ import java.util.Enumeration;
/*     */ import java.util.Vector;
/*     */ import se.lypson.common.collections.Queue;
/*     */ 
/*     */ public class SocketServer extends se.lypson.common.thread.Thread implements ILineReadObserver
/*     */ {
/*     */   private ServerSocket serverSocket;
/*     */   private ClientAcceptorThread clientAcceptorThread;
/*     */   
/*     */   private class ClientAcceptorThread extends se.lypson.common.thread.Thread
/*     */   {
/*     */     private boolean unlimitedAcceptance;
/*     */     private int numberOfClientsToAccept;
/*     */     
/*     */     public ClientAcceptorThread()
/*     */     {
/*  27 */       this.unlimitedAcceptance = true;
/*  28 */       this.numberOfClientsToAccept = -1;
/*     */     }
/*     */     
/*  31 */     public ClientAcceptorThread(int numberOfClientsToAccept) { this.unlimitedAcceptance = false;
/*  32 */       this.numberOfClientsToAccept = numberOfClientsToAccept;
/*     */     }
/*     */     
/*     */     public Object threadMain()
/*     */       throws Exception
/*     */     {
/*  38 */       while ((this.unlimitedAcceptance) || (this.numberOfClientsToAccept-- > 0)) {
/*  39 */         SocketServer.this.waitForClient();
/*     */       }
/*  41 */       return null;
/*     */     }
/*     */   }
/*     */   
/*     */   private class ClientLineReaderThread
/*     */     extends StreamLineReaderThread {
/*     */     private int clientID;
/*     */     
/*  49 */     public ClientLineReaderThread(InputStream is) { super(); }
/*     */     
/*     */     public void start(int clientID) {
/*  52 */       this.clientID = clientID;
/*  53 */       start();
/*     */     }
/*     */     
/*  56 */     public int getClientID() { return this.clientID; }
/*     */   }
/*     */   
/*     */ 
/*     */   private class ClientData
/*     */   {
/*     */     public Socket socke;
/*     */     
/*     */     public InputStream inputStream;
/*     */     
/*     */     public OutputStream outputStream;
/*     */     
/*     */     public SocketServer.ClientLineReaderThread lineReaderThread;
/*     */     
/*     */     public boolean isConnected;
/*     */     
/*     */     public ClientData(Socket socket)
/*     */       throws IOException
/*     */     {
/*  75 */       this.socke = socket;
/*  76 */       this.inputStream = socket.getInputStream();
/*  77 */       this.outputStream = socket.getOutputStream();
/*  78 */       this.lineReaderThread = new SocketServer.ClientLineReaderThread(SocketServer.this, this.inputStream);
/*  79 */       this.isConnected = true;
/*     */     }
/*     */     
/*     */     public void close() throws IOException
/*     */     {
/*  84 */       this.lineReaderThread.close();
/*  85 */       this.socke.close();
/*  86 */       this.inputStream.close();
/*  87 */       this.outputStream.close();
/*  88 */       this.isConnected = false;
/*     */     }
/*     */   }
/*     */   
/*     */   private class ClientMessage {
/*     */     public int clientID;
/*     */     public String message;
/*     */     
/*     */     public ClientMessage(int clientID, String message) {
/*  97 */       this.clientID = clientID;
/*  98 */       this.message = message;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/* 104 */   private Vector clients = new Vector();
/* 105 */   private Vector serverObservers = new Vector();
/*     */   private int port;
/*     */   private int numberOfClients;
/* 108 */   private volatile boolean stop = false;
/* 109 */   private Queue messagesToSend = new Queue();
/* 110 */   private final int ID_ALL_CLIENTS = -1;
/*     */   
/* 112 */   private String publicIpAddress = null;
/* 113 */   private final String ADDRESS_QUERY_IP = "http://www.whatismyip.org/";
/* 114 */   private final String HEADER_NAME_USERAGENT = "User-Agent";
/* 115 */   private final String HEADER_VALUE_USERAGENT = "Profile/" + System.getProperty("microedition.profiles") + " " + "Configuration/" + System.getProperty("microedition.configuration");
/*     */   
/*     */   public SocketServer(int port, int numberOfClients)
/*     */   {
/* 119 */     if ((port < 0) || (port > 65535)) {
/* 120 */       throw new IllegalArgumentException("Illegal port number: " + port + " is not in the range [0, 65535].");
/*     */     }
/* 122 */     this.port = port;
/* 123 */     this.numberOfClients = numberOfClients;
/*     */   }
/*     */   
/* 126 */   public SocketServer(int numberOfClients) { this.port = -1;
/* 127 */     this.numberOfClients = numberOfClients;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public synchronized String getPublicServerAddress()
/*     */   {
/* 135 */     if (this.publicIpAddress == null)
/*     */       try {
/* 137 */         URL url = new URL("http://www.whatismyip.org/");
/* 138 */         HttpURLConnection httpConnection = (HttpURLConnection)url.openConnection();
/* 139 */         httpConnection.addRequestProperty("User-Agent", this.HEADER_VALUE_USERAGENT);
/* 140 */         httpConnection.connect();
/* 141 */         java.io.BufferedReader in = new java.io.BufferedReader(new java.io.InputStreamReader(httpConnection.getInputStream()));
/* 142 */         this.publicIpAddress = in.readLine();
/*     */       } catch (IOException ioe) {
/* 144 */         System.out.println("Error retrieving public server address: " + ioe);
/* 145 */         ioe.printStackTrace();
/*     */       }
/* 147 */     return this.publicIpAddress;
/*     */   }
/*     */   
/* 150 */   public String getLocalServerAddress() throws IOException { java.net.InetAddress address = this.serverSocket.getInetAddress();
/* 151 */     return address.getHostName();
/*     */   }
/*     */   
/* 154 */   public int getServerPort() throws IOException { return this.serverSocket.getLocalPort(); }
/*     */   
/*     */   private synchronized int addClientForConnection(Socket socketConnection)
/*     */     throws IOException
/*     */   {
/* 159 */     ClientData client = new ClientData(socketConnection);
/* 160 */     this.clients.addElement(client);
/* 161 */     int clientIndex = this.clients.lastIndexOf(client);
/* 162 */     notifyObserversClientConnected(clientIndex);
/*     */     
/* 164 */     client.lineReaderThread.addLineReadObserver(this);
/* 165 */     client.lineReaderThread.start(clientIndex);
/* 166 */     return clientIndex;
/*     */   }
/*     */   
/*     */   private int waitForClient() throws IOException {
/* 170 */     Socket socket = this.serverSocket.accept();
/* 171 */     return addClientForConnection(socket);
/*     */   }
/*     */   
/* 174 */   private void waitForClients(int numberOfClients) throws IOException { while (numberOfClients-- > 0)
/*     */     {
/* 176 */       waitForClient();
/*     */     }
/*     */   }
/*     */   
/*     */   public synchronized void closeClientConnection(int clientID) {
/* 181 */     ClientData client = (ClientData)this.clients.elementAt(clientID);
/* 182 */     if (!client.isConnected) {
/* 183 */       return;
/*     */     }
/*     */     try
/*     */     {
/* 187 */       client.close();
/*     */ 
/*     */     }
/*     */     catch (IOException ioe)
/*     */     {
/* 192 */       ioe.printStackTrace();
/*     */     }
/*     */     finally
/*     */     {
/* 196 */       notifyObserversClientDisconnected(clientID);
/*     */     }
/*     */   }
/*     */   
/* 200 */   private synchronized void closeServer() { for (int i = this.clients.size() - 1; i >= 0; i--) {
/* 201 */       closeClientConnection(i);
/*     */     }
/*     */     try {
/* 204 */       this.serverSocket.close();
/*     */ 
/*     */     }
/*     */     catch (IOException ioe)
/*     */     {
/* 209 */       ioe.printStackTrace();
/*     */     }
/*     */   }
/*     */   
/*     */   public int getNumberOfClients() {
/* 214 */     return this.clients.size();
/*     */   }
/*     */   
/*     */   public synchronized void sendMessage(int clientID, String message) {
/* 218 */     this.messagesToSend.push(new ClientMessage(clientID, message));
/* 219 */     notify();
/*     */   }
/*     */   
/* 222 */   public synchronized void sendMessage(String message) { this.messagesToSend.push(new ClientMessage(-1, message));
/* 223 */     notify();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void transmitMessage(int clientID, String message)
/*     */   {
/* 240 */     ClientData client = (ClientData)this.clients.elementAt(clientID);
/* 241 */     if (!client.isConnected) {
/* 242 */       return;
/*     */     }
/*     */     try
/*     */     {
/* 246 */       OutputStream outputStream = client.outputStream;
/*     */       
/* 248 */       outputStream.write(message.getBytes());
/* 249 */       outputStream.write(10);
/* 250 */       outputStream.flush();
/*     */     }
/*     */     catch (IOException ioe)
/*     */     {
/* 254 */       ioe.printStackTrace();
/*     */       
/*     */ 
/* 257 */       closeClientConnection(clientID);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void transmitMessage(String message)
/*     */   {
/* 268 */     for (int i = this.clients.size() - 1; i >= 0; i--)
/* 269 */       transmitMessage(i, message);
/*     */   }
/*     */   
/*     */   public void addServerObserver(ISocketServerObserver observer) {
/* 273 */     if (observer == null) {
/* 274 */       throw new NullPointerException("Observer cannot be null.");
/*     */     }
/* 276 */     if (!this.serverObservers.contains(observer))
/* 277 */       this.serverObservers.addElement(observer);
/*     */   }
/*     */   
/* 280 */   public void removeServerObserver(ISocketServerObserver observer) { this.serverObservers.removeElement(observer); }
/*     */   
/*     */   private void notifyObserversServerStarted()
/*     */   {
/* 284 */     System.out.println("Observer information: Server started.");
/* 285 */     Enumeration observersEnum = this.serverObservers.elements();
/* 286 */     while (observersEnum.hasMoreElements()) {
/*     */       try
/*     */       {
/* 289 */         ((ISocketServerObserver)observersEnum.nextElement()).serverStartedEvent(this);
/*     */       } catch (Exception e) {}
/*     */     }
/*     */   }
/*     */   
/*     */   private void notifyObserversClientConnected(int clientID) {
/* 295 */     System.out.println("Observer information: Client connected: " + clientID + ".");
/* 296 */     Enumeration observersEnum = this.serverObservers.elements();
/* 297 */     while (observersEnum.hasMoreElements())
/*     */       try
/*     */       {
/* 300 */         ((ISocketServerObserver)observersEnum.nextElement()).clientConnectedEvent(this, clientID);
/*     */       } catch (Exception e) {}
/*     */   }
/*     */   
/*     */   private void notifyObserversClientDisconnected(int clientID) {
/* 305 */     System.out.println("Observer information: Client disconnected: " + clientID + ".");
/* 306 */     Enumeration observersEnum = this.serverObservers.elements();
/* 307 */     while (observersEnum.hasMoreElements()) {
/*     */       try
/*     */       {
/* 310 */         ((ISocketServerObserver)observersEnum.nextElement()).clientDisconnectedEvent(this, clientID);
/*     */       } catch (Exception e) {}
/*     */     }
/*     */   }
/*     */   
/*     */   private void notifyObserversMessageSent(String message) {
/* 316 */     System.out.println("Observer information: Message sent: '" + message + "'.");
/* 317 */     Enumeration observersEnum = this.serverObservers.elements();
/* 318 */     while (observersEnum.hasMoreElements())
/*     */       try
/*     */       {
/* 321 */         ((ISocketServerObserver)observersEnum.nextElement()).messageSentEvent(this, message);
/*     */       } catch (Exception e) {}
/*     */   }
/*     */   
/*     */   private void notifyObserversMessageSent(int clientID, String message) {
/* 326 */     System.out.println("Observer information: Message sent: " + clientID + ", '" + message + "'.");
/* 327 */     Enumeration observersEnum = this.serverObservers.elements();
/* 328 */     while (observersEnum.hasMoreElements())
/*     */       try
/*     */       {
/* 331 */         ((ISocketServerObserver)observersEnum.nextElement()).messageSentEvent(this, clientID, message);
/*     */       } catch (Exception e) {}
/*     */   }
/*     */   
/*     */   private void notifyObserversMessageReceived(int clientID, String message) {
/* 336 */     System.out.println("Observer information: Message received: " + clientID + ", '" + message + "'.");
/* 337 */     Enumeration observersEnum = this.serverObservers.elements();
/* 338 */     while (observersEnum.hasMoreElements()) {
/*     */       try
/*     */       {
/* 341 */         ((ISocketServerObserver)observersEnum.nextElement()).messageReceivedEvent(this, clientID, message);
/*     */       } catch (Exception e) {}
/*     */     }
/*     */   }
/*     */   
/*     */   public void lineReadEvent(Object source, String line) {
/* 347 */     notifyObserversMessageReceived(((ClientLineReaderThread)source).getClientID(), line);
/*     */   }
/*     */   
/*     */   public synchronized Object threadMain()
/*     */     throws Exception
/*     */   {
/*     */     try
/*     */     {
/* 355 */       getPublicServerAddress();
/*     */       
/* 357 */       if (this.port >= 0)
/*     */       {
/* 359 */         this.serverSocket = new ServerSocket(this.port);
/*     */       }
/*     */       else {
/* 362 */         this.serverSocket = new ServerSocket(0);
/*     */       }
/* 364 */       this.clientAcceptorThread = new ClientAcceptorThread(this.numberOfClients);
/* 365 */       this.clientAcceptorThread.start();
/*     */       
/* 367 */       notifyObserversServerStarted();
/* 368 */       System.out.println("Server address: " + getLocalServerAddress());
/* 369 */       System.out.println("Server port: " + getServerPort());
/*     */       
/*     */ 
/* 372 */       while (!this.stop)
/*     */       {
/* 374 */         while (!this.messagesToSend.isEmpty())
/*     */         {
/* 376 */           ClientMessage clientMessage = (ClientMessage)this.messagesToSend.pop();
/* 377 */           if (clientMessage.clientID != -1)
/*     */           {
/* 379 */             transmitMessage(clientMessage.clientID, clientMessage.message);
/* 380 */             notifyObserversMessageSent(clientMessage.clientID, clientMessage.message);
/*     */           }
/*     */           else
/*     */           {
/* 384 */             transmitMessage(clientMessage.message);
/* 385 */             notifyObserversMessageSent(clientMessage.message);
/*     */           }
/*     */         }
/*     */         try
/*     */         {
/* 390 */           wait();
/*     */         }
/*     */         catch (InterruptedException ie) {}
/*     */       }
/*     */     }
/*     */     catch (IOException ioe) {
/* 396 */       ioe.printStackTrace();
/* 397 */       throw ioe;
/*     */     }
/* 399 */     return null;
/*     */   }
/*     */   
/*     */   public synchronized void close() {
/* 403 */     if (!this.stop)
/*     */     {
/* 405 */       this.stop = true;
/* 406 */       notify();
/* 407 */       closeServer();
/*     */     }
/*     */   }
/*     */ }


/* Location:              /Users/Dawidejdeholm/Desktop/Skola/AI/PokerProject/PokerServerGUI/PokerServerGui.jar!/com/medicwave/socketlibrary/SocketServer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */